//	Japanese-English Dictionary Plugin v1
//  
//  Written by Chris Dahlberg 2/16/04
//  Harvey Mudd College 
//  chrisdahl1000@attbi.edu
//
//  Plugin for Oh! Text Hooker  (http://ohhara.sarang.net/ohthk
//
//  Oh! Text Hooker written by Taeho Oh 
//  ( ohhara@postech.edu, ohhara@plus.or.kr )
//  http://ohhara.sarang.net 
//

#ifndef __JE_DIC__
#define __JE_DIC__

#define UNICODE

#define OTHEXPORT __declspec(dllexport)

#include "oh_trmod.h"
#include "gdiplus.h"
#include "windows.h"
#include "commctrl.h"
#include <iostream>
#include <string>
#include <cassert>
#include <vector>
#include "word.h"
#include "dictionary.h"

using namespace std;
using namespace stdext;
using namespace Gdiplus;

const WCHAR UNI_SPACE = L' ';
const WCHAR UNI_KANA_START = L'[';
const WCHAR UNI_KANA_STOP = L']';
const WCHAR UNI_ENGDEF_START = L'/';
const WCHAR UNI_ENGDEF_STOP = L'\n';
const WCHAR UNI_KANJI_STOP = L' ';
const int READ_BUFFERSIZE = 400;
const int INITIAL_ORG_STR_LEN = 100;
const WCHAR FIRST_HIRAGANA = (WCHAR)0x3040;
const WCHAR LAST_HIRAGANA = (WCHAR)0x309F;
const WCHAR FIRST_KATAKANA = (WCHAR)0x30A0;
const WCHAR LAST_KATAKANA = (WCHAR)0x30FF;
const WCHAR FIRST_KANJI = (WCHAR)0x4E00;
const WCHAR LAST_KANJI = (WCHAR)0x9FFF;
const int TRANS_WIN_WIDTH = 640;
const int TRANS_WIN_HEIGHT = 50;

// For GDI+
GdiplusStartupInput gdiplusStartupInput;
ULONG_PTR           gdiplusToken;

HFONT g_font;

// Current position in the read buffer
int g_readIndex;
// The read buffer itself
LPWSTR g_readBuffer;


static HINSTANCE hInst;

// The dictionary being used.
Dictionary dict;

// Holds memory allocated for the original string recieved from OTH
WCHAR* orgStr;
int orgStrLen;

// Buffer readers

// Extract one symbol from the read buffer.
WCHAR readWChar(HANDLE hfile);
// Read from the file into the buffer.
bool readBuffer(HANDLE hfile, LPWSTR buffer, int bufferSize);
// Allocate memory and read the first batch into the global read buffer.
void initReadBuffer(HANDLE hfile);
// Deallocate the read buffer.
void closeReadBuffer();
// Skip over all the adjacent sym symbols in the read buffer.
bool clearSymbol(HANDLE hfile, WCHAR sym);
// Read and parse a word structure from a EDICT formatted dictionary file.
word readWord(HANDLE hfile);
// Read a string from the read buffer until the stop WCHAR is reached.
bool getString(wstring* str, HANDLE hfile, WCHAR stop);

// Misc Functions
// Load the dictionary.
vector<word> openDict(LPCWSTR dictpath, Dictionary* dict);

// The tokenize functions split tokens off of the buffer while the WCHARs are 
// either inside or outside the range specified.
wstring tokenizeWhileOutside(LPCWSTR srcBuffer, 
							 int* currentPos,
							 WCHAR lower,
							 WCHAR upper);
wstring tokenizeWhileInside(LPCWSTR srcBuffer,
							int* currentPos,
							WCHAR lower,
							WCHAR upper);
// Window procedure for the translation window.
LRESULT CALLBACK transWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
// Function to create the translation window.
HWND* createTransWindow(HINSTANCE hInst);

// Plugin functions

OTHEXPORT int OTHCALL OthTransInit(OTHTR *pTR);
OTHEXPORT int OTHCALL OthTransSetting(OTHTR *pTR);
OTHEXPORT int OTHCALL OthTransCopySetting(OTHTR *pDestTR, OTHTR *pSrcTR);
OTHEXPORT int OTHCALL OthTranslate(OTHTR *pTR, LPCWSTR lpStr, int iLen);
OTHEXPORT int OTHCALL OthTransDestroy(OTHTR *pTR);
BOOL APIENTRY DllMain( HINSTANCE hModule,                       
					   DWORD  ul_reason_for_call, 
                       LPVOID lpReserved);


#endif